@@ -1,7 +1,7 @@
 # Functions to manipulate Python binary distribution archives.
 #
 # Author: Peter Odding <peter.odding@paylogic.eu>
-# Last Change: June 18, 2014
+# Last Change: June 25, 2014
 # URL: https://github.com/paylogic/pip-accel
 
 """
@@ -21,6 +21,7 @@ import pipes
 import re
 import shutil
 import subprocess
+import sys
 import tarfile
 import tempfile
 import time
@@ -188,7 +189,7 @@ def transform_binary_dist(archive_path, prefix='/usr'):
                 yield member, handle
     archive.close()
 
-def install_binary_dist(members, prefix, python='/usr/bin/python', enable_workarounds=True):
+def install_binary_dist(members, prefix, python='/usr/bin/python', virtualenv_compatible=True):
     """
     Install a binary distribution created with ``python setup.py bdist`` into
     the given prefix (a directory like ``/usr``, ``/usr/local`` or a virtual
@@ -202,26 +203,39 @@ def install_binary_dist(members, prefix, python='/usr/bin/python', enable_workar
     :param python: The pathname of the Python executable to use in the shebang
                    line of all executable Python scripts inside the binary
                    distribution.
-    :param enable_workarounds: Enable workarounds to make the resulting filenames
-                               compatible with virtual environments.
+    :param virtualenv_compatible: Enable workarounds to make the resulting
+                                  filenames compatible with virtual
+                                  environments.
     """
-    # TODO Rename enable_workarounds -> virtualenv_compatible?
     # TODO This is quite slow for modules like Django. Speed it up! Two choices:
     #  1. Run the external tar program to unpack the archive. This will
     #     slightly complicate the fixing up of hashbangs.
     #  2. Using links? The plan: We can maintain a "seed" environment under
     #     $PIP_ACCEL_CACHE and use symbolic and/or hard links to populate other
     #     places based on the "seed" environment.
+    on_debian = os.path.exists('/etc/debian_version')
+    module_search_path = set(map(os.path.normpath, sys.path))
     for member, from_handle in members:
         pathname = member.name
-        # Some binary distributions include C header files (see for example the
-        # greenlet package) however the subdirectory of include/ in a virtual
-        # environment is a symbolic link to a subdirectory of /usr/include/ so
-        # we should never try to install C header files inside the directory
-        # pointed to by the symbolic link. Instead we implement the same
-        # workaround that pip uses to avoid this problem.
-        if enable_workarounds:
+        if virtualenv_compatible:
+            # Some binary distributions include C header files (see for example
+            # the greenlet package) however the subdirectory of include/ in a
+            # virtual environment is a symbolic link to a subdirectory of
+            # /usr/include/ so we should never try to install C header files
+            # inside the directory pointed to by the symbolic link. Instead we
+            # implement the same workaround that pip uses to avoid this
+            # problem.
             pathname = re.sub('^include/', 'include/site/', pathname)
+        if on_debian and '/site-packages/' in pathname:
+            # On Debian based system wide Python installs the /site-packages/
+            # directory is not in Python's module search path while
+            # /dist-packages/ is. We try to be compatible with this.
+            match = re.match('^(.+?)/site-packages', pathname)
+            if match:
+                site_packages = os.path.normpath(os.path.join(prefix, match.group(0)))
+                dist_packages = os.path.normpath(os.path.join(prefix, match.group(1), 'dist-packages'))
+                if dist_packages in module_search_path and site_packages not in module_search_path:
+                    pathname = pathname.replace('/site-packages/', '/dist-packages/')
         pathname = os.path.join(prefix, pathname)
         directory = os.path.dirname(pathname)
         if not os.path.isdir(directory):
